/**
 * Copyright (c) 2021 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#include "turn.h"

pokerturn_t pokerTurnGet(poker_t *poker, uint8_t playerIndex) {
  pokerplayer_t *player;
  float confidence;
  player = poker->players + playerIndex;

  // Can the player do anything?
  if(player->state & (POKER_PLAYER_STATE_FOLDED | POKER_PLAYER_STATE_OUT)) {
    return pokerTurnOut(poker, playerIndex);
  }

  // Now we basically need to determine the AI's action here. The AI really only
  // needs to do one thing, decide whether or not they want to Bet or Not.
  // In future I may make checking optional, but for now there's really no 
  // reason not to.

  // Get the current winning hand.
  pokerplayerwinning_t winning;
  pokerWinnerPlayerGet(&poker->dealer, player, &winning);

  // Now let's determine the "base confidence". This is basically how good the
  // hand is relative to a completely equal footing/player field.
  switch(winning.type) {
    case POKER_WINNING_TYPE_ROYAL_FLUSH:
      confidence = POKER_TURN_CONFIDENCE_ROYAL_FLUSH;
      break;
    case POKER_WINNING_TYPE_STRAIGHT_FLUSH:
      confidence = POKER_TURN_CONFIDENCE_STRAIGHT_FLUSH;
      break;
    case POKER_WINNING_TYPE_FOUR_OF_A_KIND:
      confidence = POKER_TURN_CONFIDENCE_FOUR_OF_A_KIND;
      break;
    case POKER_WINNING_TYPE_FULL_HOUSE:
      confidence = POKER_TURN_CONFIDENCE_FULL_HOUSE;
      break;
    case POKER_WINNING_TYPE_FLUSH:
      confidence = POKER_TURN_CONFIDENCE_FLUSH;
      break;
    case POKER_WINNING_TYPE_STRAIGHT:
      confidence = POKER_TURN_CONFIDENCE_STRAIGHT;
      break;
    case POKER_WINNING_TYPE_THREE_OF_A_KIND:
      confidence = POKER_TURN_CONFIDENCE_THREE_OF_A_KIND;
      break;
    case POKER_WINNING_TYPE_TWO_PAIR:
      confidence = POKER_TURN_CONFIDENCE_TWO_PAIR;
      break;
    case POKER_WINNING_TYPE_PAIR:
      confidence = POKER_TURN_CONFIDENCE_PAIR;
      break;
    default:
      confidence = POKER_TURN_CONFIDENCE_HIGH_CARD;
      break;
  }
  
  // Now we have a base confidence, let's figure out how good our hand is based
  // on our high card. This will decide whether or not our hand is as strong as
  // it seems on its base level

  // Well, now they have decided to "not bet", or really they think that their
  // cards are not worth the effort. But if they CAN check, they will.
  if(pokerTurnCanPlayerCheck(poker, playerIndex)) {
    return pokerTurnCheck(poker, playerIndex);
  }

  // Nothing worth doing, so let's just fold here.
  return pokerTurnFold(poker, playerIndex);
}

void pokerTurnAction(poker_t *poker, pokerplayer_t *player, pokerturn_t *turn) {
  switch(turn->type) {
    case POKER_TURN_TYPE_BET:
    case POKER_TURN_TYPE_CALL:
    case POKER_TURN_TYPE_ALL_IN:
      pokerBetPlayer(&poker->bet, player, turn->chips);
      break;

    case POKER_TURN_TYPE_CHECK:
      pokerBetPlayer(&poker->bet, player, 0);
      break;

    case POKER_TURN_TYPE_FOLD:
      player->state |= POKER_PLAYER_STATE_FOLDED;
      break;
  }
  
  player->state |= POKER_PLAYER_STATE_ROUND_MOVE;
}

pokerturn_t pokerTurnOut(poker_t *poker, uint8_t player) {
  pokerturn_t turn;
  turn.type = POKER_TURN_TYPE_OUT;
  return turn;
}

pokerturn_t pokerTurnFold(poker_t *poker, uint8_t player) {
  pokerturn_t turn;
  turn.type = POKER_TURN_TYPE_FOLD;
  return turn;
}

pokerturn_t pokerTurnCheck(poker_t *poker, uint8_t player) {
  pokerturn_t turn;
  turn.type = POKER_TURN_TYPE_CHECK;
  return turn;
}

pokerturn_t pokerTurnCall(poker_t *poker, uint8_t playerIndex) {
  pokerturn_t turn;
  pokerplayer_t *player;
  player = poker->players + playerIndex;
  turn = pokerTurnRaise(
    poker, playerIndex, poker->bet.currentBet - player->currentBet
  );
  turn.type = (
    turn.type == POKER_TURN_TYPE_ALL_IN ?
    POKER_TURN_TYPE_CALL_ALL_IN :
    POKER_TURN_TYPE_CALL
  );
  return turn;
}

pokerturn_t pokerTurnRaise(poker_t *poker, uint8_t playerIndex, int32_t chips) {
  pokerturn_t turn;
  pokerplayer_t *player;
  player = poker->players + playerIndex;

  if(player->chips <= chips) {
    turn.chips = player->chips;
    turn.type = POKER_TURN_TYPE_ALL_IN;
  } else {
    turn.chips = chips;
    turn.type = POKER_TURN_TYPE_BET;
  }

  return turn;
}

bool pokerTurnCanPlayerCheck(poker_t *poker, uint8_t playerIndex) {
  return (poker->players + playerIndex)->currentBet >= poker->bet.currentBet;
}